-----------------------------------------------------------------------------------------------
-- Client Lua Script for SSToolkitEx
-- Copyright (c) NCsoft. All rights reserved
-- Written By: The Weasel (Entity)
-----------------------------------------------------------------------------------------------
require "Apollo"
require "ChatSystemLib"
require "GameLib"
require "GroupLib"
require "Window"

-----------------------------------------------------------------------------------------------
-- SSToolkitEx Module Definition
-----------------------------------------------------------------------------------------------
local SSToolkitEx = {} 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SSToolkitEx:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	
	--Version
    self.Version = 1.05
	self.NeedReset = false --Does this build need a full options reset since previous build?
	self.NeedPositionReset = false --Does this build need all window positions reset since previous build?
	self.NeedSettingsOnlyPositionReset = true --Does this build only need the settings window position reset since previous build?

	self.UIUnlocked = false
	self.PopTimer = Apollo.GetTickCount()
	self.iTimeRemaining = 0
	self.VPUnits = {}
	self.IgniteUnits = {}
	self.units = {}
	self.SIGIL = {}
	
    return o
end

function SSToolkitEx:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- SSToolkitEx OnLoad
-----------------------------------------------------------------------------------------------
function SSToolkitEx:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SSToolkitEx.xml")
	Apollo.LoadSprites("WeaselSprite.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SSToolkitEx OnDocLoaded
-----------------------------------------------------------------------------------------------
function SSToolkitEx:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SSToolkitExForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.ICO = Apollo.LoadForm(self.xmlDoc, "ICO", "InWorldHudStratum", self)
		self.OverlayIgnite = Apollo.LoadForm(self.xmlDoc, "OverlayIgnite", "FixedHudStratumLow", self)
		self.OverlayLists = Apollo.LoadForm(self.xmlDoc, "OverlayLists", "FixedHudStratumLow", self)
		self.OverlaySound = Apollo.LoadForm(self.xmlDoc, "SoundList", nil, self)
		
		self.opt = {}
		self:RegisterGUIElements()
		
	    self.wndMain:Show(false, true)
	    self.ICO:Show(true, true)
		self.OverlayIgnite:Show(true, true)
		self.OverlayLists:Show(true, true)
		
		-- Register handlers for events, slash commands and timer, etc.
		Apollo.RegisterSlashCommand("sst",	 	"OnSSToolkitExOn", 	self)
		Apollo.RegisterSlashCommand("SST", 		"OnSSToolkitExOn", 	self)
		Apollo.RegisterSlashCommand("ss",	 	"OnSSToolkitExOn", 	self)
		Apollo.RegisterSlashCommand("SS", 		"OnSSToolkitExOn", 	self)
		Apollo.RegisterSlashCommand("vp",	 	"OnSSToolkitExOn", 	self)
		Apollo.RegisterSlashCommand("VP",	 	"OnSSToolkitExOn", 	self)

		self.timer = ApolloTimer.Create(0.25, true, "OnTimer", self) --Lookup frequency
		self.timer2 = ApolloTimer.Create(0.5, true, "OnTimerIgniteCleanup", self) --Ignite list cleanup frequency
		self.timer3 = ApolloTimer.Create(0.25, true, "OnTimerSigil", self)	

		Apollo.RegisterEventHandler("NextFrame", "OnFrame", self) --Draw frequency (NextFrame)
		Apollo.RegisterEventHandler("CombatLogDamage", "OnCombatLogDamage", self) --To track NewIgnite
		Apollo.RegisterEventHandler("CombatLogDeflect", "OnCombatLogDeflect", self) --To track ignite deflect
		Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self) --To remove dead units from IgniteUnits
		Apollo.RegisterEventHandler("CombatLogDeath", "OnCombatLogDeath", self) --To remove all ignite units on death
		Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)

		--Add to Interface menu
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("OpenSSToolkit", "OnSSToolkitExOn", self)

		-- Do additional Addon initialization here
		-- Load
		self.opt.Version:SetText("v " .. self.Version)
		
		if self.OptEnabled 	~= nil then	self.opt.VPIconEnabled:SetCheck(self.OptEnabled) 	end
		if self.OptTimer 	~= nil then self.opt.Timer:SetCheck(self.OptTimer) 		end
		if self.OptSizeVP 	~= nil then
			self.opt.SizeVP = self.OptSizeVP
		else
			self.opt.SizeVP  = 20
		end
		self.opt.Size:SetValue(self.opt.SizeVP)
		
		if self.OptIgniteMark 	~= nil then self.opt.IgniteMark:SetCheck(self.OptIgniteMark) end
		if self.OptXY 			~= nil then self.opt.XY:SetCheck(self.OptXY) end
		if self.OptIgniteTimer 	~= nil then self.opt.IgniteTimer:SetCheck(self.OptIgniteTimer) end
		if self.OptSizeIgnite 	~= nil then
			self.opt.SizeIgnite = self.OptSizeIgnite
		else
			self.opt.SizeIgnite = 20
		end
		self.opt.IgniteSize:SetValue(self.opt.SizeIgnite)
		
		if self.OptIgniteFinalAlert 		~= nil then self.opt.IgniteFinalAlert:SetCheck(self.OptIgniteFinalAlert) end
		if self.OptIgniteDeflectAlert 		~= nil then self.opt.IgniteDeflectAlert:SetCheck(self.OptIgniteDeflectAlert) end
		if self.OptIgniteSurgeAlert 		~= nil then self.opt.IgniteSurgeAlert:SetCheck(self.OptIgniteSurgeAlert) end
		
		if self.OptIgniteFinalSoundOpt 		~= nil then self.opt.IgniteFinalSound:SetCheck(self.OptIgniteFinalSoundOpt) end
		if self.OptIgniteDeflectSoundOpt 	~= nil then self.opt.IgniteDeflectSound:SetCheck(self.OptIgniteDeflectSoundOpt) end
		if self.OptIgniteSurgeSoundOpt 		~= nil then self.opt.IgniteSurgeSound:SetCheck(self.OptIgniteSurgeSoundOpt) end
		
		if self.OptIgniteList 				~= nil then self.opt.IgniteList:SetCheck(self.OptIgniteList) end
		
		--Set position of Ignite List
		if self.OptPosIgniteUI ~= nil then
			self.IgniteUI:SetAnchorOffsets(
			self.OptPosIgniteUI.Left,
			self.OptPosIgniteUI.Top,
			self.OptPosIgniteUI.Right,
			self.OptPosIgniteUI.Bottom)
		end
		
		--Set position of Configuration window
		if self.OptPosConfigUI ~= nil then
			self.wndMain:SetAnchorOffsets(
			self.OptPosConfigUI.Left,
			self.OptPosConfigUI.Top,
			self.OptPosConfigUI.Right,
			self.OptPosConfigUI.Bottom)
		end
		 
		--Set position of Ignite Alert
		if self.OptPosIgniteAlert ~= nil then
			self.opt.IgniteAlert:SetAnchorOffsets(
			self.OptPosIgniteAlert.Left,
			self.OptPosIgniteAlert.Top,
			self.OptPosIgniteAlert.Right,
			self.OptPosIgniteAlert.Bottom)
		end
		
		--Set position of VPXY
		if self.OptPosXY ~= nil then
			self.VPUI:SetAnchorOffsets(
			self.OptPosXY.Left,
			self.OptPosXY.Top,
			self.OptPosXY.Right,
			self.OptPosXY.Bottom)
		end
		
		self:PopulateSounds()
		if self.OptIgniteFinalSound		 ~= nil then self.opt.SoundFinal:SetText(self.OptIgniteFinalSound) 		end
		if self.OptIgniteDeflectSound	 ~= nil then self.opt.SoundDeflect:SetText(self.OptIgniteDeflectSound) 	end
		if self.OptIgniteSurgeSound		 ~= nil then self.opt.SoundSurge:SetText(self.OptIgniteSurgeSound) 		end
		
		self.VPUI:Show(false, true)
		self.OverlaySound:Show(false, true)
		
		--Initialize
		self:OnSliderChange()
		self:OnVPXY()
		self:OnIgniteBG()
		self:Ignite("clear")
		self:OnIgnitListClick()
		self:OnMenuHide()
		
		if not self.opt.VPIconEnabled:IsChecked() then self:EnableHide() end -- Hide controls after loading settings if main option is not enabled
	end
end

-----------------------------------------------------------------------------------------------
-- SSToolkitEx Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
-- on SlashCommand
function SSToolkitEx:OnSSToolkitExOn()
	self.OverlaySound:Show(false, true) --Hide sound form
	self.wndMain:Invoke() --Show main form
end

function SSToolkitEx:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "SSToolkitEx", {"OpenSSToolkit", "", "WeaselSprite:SmallWeasel"})
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Realm, "SSToolkitEx v" .. self.Version .. " - Various tools for Spellslingers (more options to come).\n\"/ss\" for options.  -The Weasel.")
end

-- on timer
function SSToolkitEx:OnTimer()
	--self:CheckVP()
	if Apollo.GetTickCount() - self.PopTimer > 1500 then self:Ignite("clear") end
end

function SSToolkitEx:OnFrame()
	self.ICO:DestroyAllPixies()
	if self.opt.VPIconEnabled:IsChecked() == true then --Track VP with icons
		--for key,val in pairs(self.VPUnits) do
			--VP Icon and Timer
		--	local aScreenPos = GameLib.GetUnitScreenPosition(val.unit)
		--	if aScreenPos and aScreenPos.bOnScreen and val.unit:IsValid() then
		--		local ModTime = self.VPUnits[val.unitID].iTime - (os.clock() - self.VPUnits[val.unitID].TimerDiff)
		--		if ModTime < 0 then ModTime = 0 end
		--		local DisplayTime = string.format("%3.1f", tostring(ModTime))
		--		if self.opt.Timer:IsChecked() ~= true then DisplayTime = nil end
		--		self:CreateUnitIcon({tUnit = val.unit, iSize = self.opt.SizeVP, strText = DisplayTime, strFont = self.opt.VPfont, strSprite = "IconSprites:Icon_SkillSpellslinger_void_pact", ScreenPos = aScreenPos})
		--	end
		--end
		
		for key,val in pairs(self.SIGIL) do
			--VP Icon and Timer
			local aScreenPos = GameLib.GetUnitScreenPosition(val.unit)
			if aScreenPos and aScreenPos.bOnScreen and val.unit:IsValid() then
				local ModTime = self.SIGIL[val.unitID].iTime - (os.clock() - self.SIGIL[val.unitID].TimerDiff)
				if ModTime < 0 then ModTime = 0 end
				local DisplayTime = string.format("%3.1f", tostring(ModTime))
				if self.opt.Timer:IsChecked() ~= true then DisplayTime = nil end
				self:CreateUnitIcon({tUnit = val.unit, iSize = self.opt.SizeVP, strText = DisplayTime, strFont = self.opt.VPfont, strSprite = "IconSprites:Icon_SkillSpellslinger_sigil_of_recovery", ScreenPos = aScreenPos})
			end
		end
	end

	for key,val in pairs(self.IgniteUnits) do
		--Ignite List
		local tock = val.ticks / 8
		local TimerOffset = os.clock() - val.TickTime
		local DisplayTime = string.format("%3.1f", tostring(val.ticks + 1 - TimerOffset))
		if tock < 0 then
			tock = 0
			DisplayTime = "?"
		end
		val.listunit:FindChild("Time"):SetText(DisplayTime)
		val.listunit:FindChild("ProgressBar"):SetProgress(tock, .125)
			
		--Ignite Icon and Timer
		local aScreenPos = GameLib.GetUnitScreenPosition(val.unit)
		if aScreenPos and aScreenPos.bOnScreen and val.unit:IsValid() then
			if self.opt.IgniteTimer:IsChecked() ~= true then DisplayTime = nil end
			if self.opt.IgniteMark:IsChecked() == true then self:CreateUnitIcon({tUnit = val.unit, iSize = self.opt.SizeIgnite, strText = DisplayTime, strFont = self.opt.Ignitefont, strSprite = "IconSprites:Icon_SkillSpellslinger_ignite", ScreenPos = aScreenPos}) end
		end
	end

end

function SSToolkitEx:OnCombatLogDeflect(evt)
	if not evt then return end
    if evt.unitCaster ~= nil and evt.unitCaster == GameLib.GetPlayerUnit() then
		local LogSpellId = evt.splCallingSpell:GetBaseSpellId()
		if LogSpellId == 25046 or LogSpellId == 34284 then --Final ignite ticks
			self:Ignite("finaltick", evt.unitTarget:GetName())
			self:RemoveIgniteUnit(evt.unitTarget:GetId()) --Old ignite unit
		end
		if LogSpellId == 53160 or LogSpellId == 20717 or LogSpellId == 20716 then --DoT ignite ticks
			local NewIgniteID = evt.unitTarget:GetId()
			if self.IgniteUnits[NewIgniteID] ~= nil then
				if LogSpellId == 20717 and evt.splCallingSpell:GetTier() ~= 9 then self:TickTock(NewIgniteID) end--Tick an existing list unit
				if LogSpellId == 53160 then self:TickTock(NewIgniteID) end--Tick an existing list unit
			else
				self:Ignite("deflect", evt.unitTarget:GetName()) -- Delfected by unit we do not know.
			end
		end
	end
end

function SSToolkitEx:OnCombatLogDamage(evt)
	if not evt then return end
	if evt.unitTarget == nil then return end --Prevent crash on an odd call
	if evt.bTargetKilled or evt.nOverkill > 0 then --Did a unit just die?
		self:RemoveIgniteUnit(evt.unitTarget:GetId()) --Old ignite unit
	end
    if evt.unitCaster ~= nil and evt.unitCaster == GameLib.GetPlayerUnit() then --Check for PlayerUnit source
		local LogSpellId = evt.splCallingSpell:GetBaseSpellId()
		local NewIgniteID = evt.unitTarget:GetId()
		if LogSpellId == 20716 and self.IgniteUnits[NewIgniteID] == nil then --Non surged ignite(missed rotation)
			self:Ignite("no-surge", evt.unitTarget:GetName())
		end
		if LogSpellId == 25046 or LogSpellId == 34284 then --Final ignite ticks
			self:Ignite("finaltick", evt.unitTarget:GetName())
			self:RemoveIgniteUnit(evt.unitTarget:GetId()) --Old ignite unit
		end
		if LogSpellId == 53160 or LogSpellId == 20717 then --DoT ignite ticks
			if self.IgniteUnits[NewIgniteID] == nil then
				self:AddIgniteUnit(NewIgniteID, evt.unitTarget, evt.unitTarget:GetName()) --New ignite unit
			else
				if LogSpellId == 20717 and evt.splCallingSpell:GetTier() ~= 9 then
					self:TickTock(NewIgniteID)--Tick an existing list unit
				end
				if LogSpellId == 53160 then
					self:TickTock(NewIgniteID)--Tick an existing list unit
				end
			end
		end
	end
end


function SSToolkitEx:OnEnteredCombat(unit, bInCombat)
	if not unit then return end
	if bInCombat then
		if self.units[unit:GetId()] == nil then
			self.units[unit:GetId()] = unit
		end
	else 
		self.units[unit:GetId()] = nil
		self.SIGIL[unit:GetId()] = nil		
	end
end

function SSToolkitEx:OnTimerSigil()
	for i,lunit in next, self.units do
		if lunit then
			local tAllBuffs = self:getAllBuffs(lunit)
			local tNewDebuffs = tAllBuffs
			local bsigil = false
			for nIdBuff,tNew in next, tNewDebuffs do
				if tNew.nSpellId == 88338 or tNew.nSpellId == 88119
					or tNew.nSpellId == 88337 or tNew.nSpellId == 88020 
					or tNew.nSpellId == 88120 or tNew.nSpellId == 88339
					or tNew.nSpellId == 88121 or tNew.nSpellId == 88340 
					or tNew.nSpellId == 88122 or tNew.nSpellId == 88341 
					or tNew.nSpellId == 88123 or tNew.nSpellId == 88342
					or tNew.nSpellId == 88124 or tNew.nSpellId == 88343
					or tNew.nSpellId == 88125 or tNew.nSpellId == 88344
					or tNew.nSpellId == 88126 or tNew.nSpellId == 88345 
				then
					bsigil = true
					self.SIGIL[lunit:GetId()] = {unit = lunit, iTime = tNew.fTimeRemaining, TimerDiff = os.clock(), unitID = lunit:GetId()}
				end
			end
			if bsigil == false then
				self.SIGIL[lunit:GetId()] = nil
			end
		end
	end
end


function SSToolkitEx:getAllBuffs(tUnit)
    local r = {}
    if tUnit then
        local tAllBuffs = tUnit:GetBuffs()["arHarmful"]
        if tAllBuffs then
			for _,obj in next, tAllBuffs do
        		local nSpellId = obj.splEffect:GetId()
				if nSpellId then
					r[obj.idBuff] = {
						nCount = obj.nCount,
						nSpellId = nSpellId,
						fTimeRemaining = obj.fTimeRemaining,
					}
				end
			end
        end
    end
    return r
end


function SSToolkitEx:OnUnitDestroyed(unit)
	local DeadId = unit:GetId()
	if self.IgniteUnits[DeadId] ~= nil then self:RemoveIgniteUnit(DeadId) end --Old ignite unit
	if self.units[DeadId] ~= nil then self:RemoveSigilUnit(DeadId) end
end

function SSToolkitEx:OnCombatLogDeath(unit) --Player died, reset ignite units
	for key,val in pairs(self.IgniteUnits) do
		self:RemoveIgniteUnit(val.unitID)
	end
end

function SSToolkitEx:OnTimerIgniteCleanup()
	for key,val in pairs(self.IgniteUnits) do
		if val.unit:IsValid() ~= true or val.unit:IsDead() ~= false then self:RemoveIgniteUnit(val.unitID) end --Remove invalid/dead units
		if os.clock() - val.TickTime > 2 then
			self:RemoveIgniteUnit(val.unitID)
		end --Remove unit that has not seen a tick in over 2 seconds.
	end
end

function SSToolkitEx:OnIgnitListClick()
	if self.opt.IgniteList:IsChecked() then
		self.IgniteUI:Show(true)
	else
		self.IgniteUI:Show(false)
	end
end

function SSToolkitEx:TickTock(TickID)
	self.IgniteUnits[TickID].ticks = self.IgniteUnits[TickID].ticks - 1
	self.IgniteUnits[TickID].TickTime = os.clock()
end

function SSToolkitEx:AddIgniteUnit(iID, iUnit, sUnitName)
	self.IgniteUnits[iID] = {unit = iUnit, listunit =  Apollo.LoadForm(self.xmlDoc, "ListItem", self.IgniteList, self), ticks = 8, unitID = iID, TickTime = os.clock()} --Add unit to ignite table
	self.IgniteUnits[iID].listunit:FindChild("ProgressBar"):SetProgress(1)
	self.IgniteUnits[iID].listunit:FindChild("Unit"):SetText(sUnitName)
	self.IgniteList:ArrangeChildrenVert(0)
	--self:TickTock(iID)
end

function SSToolkitEx:RemoveIgniteUnit(iID)
	if self.IgniteUnits[iID] == nil then return end
	self.IgniteUnits[iID].listunit:Destroy() --Destroy list item
	self.IgniteUnits[iID] = nil  --Remove unit from ignite table
	self.IgniteList:ArrangeChildrenVert(0)
end

function SSToolkitEx:RemoveSigilUnit(iID)
	--self.SIGIL[iID].listunit:Destroy() --Destroy list item
	--self.SIGIL:ArrangeChildrenVert(0)
	self.units[iID] = nil
	self.SIGIL[iID] = nil  --Remove unit from ignite table
end

function SSToolkitEx:Ignite(sStatus, sSubtext)
	self.PopTimer = Apollo.GetTickCount()
	if self.UIUnlocked then --User is moving UI elements
		self.opt.IgniteText:SetText("Ignite Alert")
		self.opt.IgniteSubText:SetText("Unlocked for placement")
		return
	end
	if sStatus == "deflect" then
		if self.opt.IgniteDeflectAlert:IsChecked() == true then
			self.opt.IgniteText:SetText("Deflected")
			self.opt.IgniteSubText:SetText(sSubtext)
			self.opt.IgniteAlert:Show(true, true) --Show the deflected alert
		end
		if self.opt.IgniteDeflectSound:IsChecked() == true then
			self:SoundPlay(self.opt.SoundDeflect:GetText())
		end
	elseif sStatus == "no-surge" then
		if self.opt.IgniteSurgeAlert:IsChecked() == true then
			self.opt.IgniteText:SetText("NOT SURGED")
			self.opt.IgniteSubText:SetText(sSubtext)
			self.opt.IgniteAlert:Show(true, true) --Show the non surge alert
		end
		if self.opt.IgniteSurgeSound:IsChecked() == true then
			self:SoundPlay(self.opt.SoundSurge:GetText())
		end
	elseif sStatus == "finaltick" then
		if self.opt.IgniteFinalAlert:IsChecked() == true then
			self.opt.IgniteText:SetText("Final tick")
			self.opt.IgniteSubText:SetText(sSubtext)
			self.opt.IgniteAlert:Show(true, true) --Show the non surge alert
		end
		if self.opt.IgniteFinalSound:IsChecked() == true then
			self:SoundPlay(self.opt.SoundFinal:GetText())
		end
	elseif sStatus == "clear" then
		self.opt.IgniteAlert:Show(false, true) --Hide the alert window
		self.opt.IgniteText:SetText("")
		self.opt.IgniteSubText:SetText("")
	end
end

function SSToolkitEx:SoundPlay(soundname)
	if Sound[soundname] == nil then return end
	Sound.Play(Sound[soundname])
end

function SSToolkitEx:CheckVP()
	local iMembers = GroupLib.GetMemberCount()
	local yVP = 1
	local xVP = 0
	if iMembers == 0 then
		local CheckUnit = GameLib.GetPlayerUnit() --Not in a party, so only test player unit
		if CheckUnit then
			local ID = CheckUnit:GetId()
			if self:IsBuffOnTarget(CheckUnit, "buff", "id", 38773) or self:IsBuffOnTarget(CheckUnit, "buff", "id", 47117) then --check for VP buff
				xVP = xVP + 1						--Increment VP found
				self.VPUnits[ID] = {unit = CheckUnit, iTime = self.iTimeRemaining, TimerDiff = os.clock(), unitID = ID}
			else
				self.VPUnits[ID] 		= nil 				--Remove unit from our table since they do not have the buff
			end
		end
		self.VPText:SetText(tostring(xVP) .. "/" .. tostring(yVP) .. " VP")
		return
	else
		yVP = 0
		for i=1, GroupLib.GetMemberCount() do --In a party, so iterate through members
			local CheckUnit = GroupLib.GetUnitForGroupMember(i)
			if CheckUnit and CheckUnit:IsDead() ~= true then
				yVP = yVP + 1
				local ID = CheckUnit:GetId()
				if self:IsBuffOnTarget(CheckUnit, "buff", "id", 38773) or self:IsBuffOnTarget(CheckUnit, "buff", "id", 47117) then --check for VP buff
					xVP = xVP + 1						--Increment VP found
					self.VPUnits[ID] = {unit = CheckUnit, iTime = self.iTimeRemaining, TimerDiff = os.clock(), unitID = ID}
				else
					self.VPUnits[ID] 		= nil 				--Remove unit from our table since they do not have the buff
				end
			end
		end
		self.VPText:SetText(tostring(xVP) .. "/" .. tostring(yVP) .. " VP")
	end
end

function SSToolkitEx:CreateUnitIcon(t)
	if t.tUnit == nil then return end
	if t.iSize == nil then return end
	t.ScreenPos		= t.ScreenPos	or GameLib.GetUnitScreenPosition(t.tUnit)
	t.strText 		= t.strText 	or nil
	t.strFont 		= t.strFont 	or nil
	t.strSprite 	= t.strSprite 	or nil
	t.cr 			= t.cr 			or {a=1,r=1,g=1,b=1}
	t.crText 		= t.crText 		or {a=1,r=1,g=1,b=1}

	local icoSize, xOffSet, yOffSet = t.iSize / 2, 0, 0
	local _flagsText = {DT_CENTER = true, DT_VCENTER = true}
	
	--Add logic here to change x and y offset to move icons if more than one is to be drawn
	local _sTextPos = "center"
	if _sTextPos == "down"	then yOffSet = yOffSet - (icoSize) end
	if _sTextPos == "up"	then yOffSet = yOffSet + (icoSize) end
	if _sTextPos == "left"	then xOffSet = xOffSet + (icoSize) end
	if _sTextPos == "right"	then xOffSet = xOffSet - (icoSize) end

	local _loc = {
		fPoints = {0,0,0,0},
		nOffsets = {t.ScreenPos.nX-xOffSet-icoSize, t.ScreenPos.nY-yOffSet-icoSize, t.ScreenPos.nX-xOffSet+icoSize, t.ScreenPos.nY-yOffSet+icoSize}
	}

	local nIcon = self.ICO:AddPixie({strText = t.strText, strFont = t.strFont, flagsText = _flagsText, strSprite = t.strSprite, loc = _loc, cr = t.cr, crText = t.crText})
end

function SSToolkitEx:IsBuffOnTarget(ThisUnit, BuffDebuff, NameId, ThisNameId)
	local aCheckBuffs
	local aBuffs = ThisUnit:GetBuffs()
	if BuffDebuff == "buff" then
		aCheckBuffs = aBuffs.arBeneficial
	elseif BuffDebuff == "debuff" then
		aCheckBuffs = aBuffs.arHarmful
	else
		return "error"
	end
	for key,value in pairs(aCheckBuffs) do
		local spell = value.splEffect
		local testme --Testing name or ID?
		if NameId == "id" then
			testme = spell:GetBaseSpellId()
		elseif NameId == "name" then
			testme = spell:GetName()
		else
			return "error"
		end
		if testme == ThisNameId then --Check conditions
			self.iTimeRemaining = value.fTimeRemaining
			return true 
		end
	end
	return false --Buff/Debuff was NOT found on unit.
end

--Future feature to scale on distance
function SSToolkitEx:CalculateDistance(vector)
	return math.sqrt(math.pow(vector.x, 2) + math.pow(vector.y, 2) + math.pow(vector.z, 2))
end

function SSToolkitEx:DistToLoc(loc, unit)
	local unitPos
	if type(unit) == "table" then
		unitPos = unit
	else
		unitPos = unit:GetPosition()
	end
	local distance = self:CalculateDistance(Vector3.New(loc.x - unitPos.x, loc.y - unitPos.y, loc.z - unitPos.z))
	if distance then 
		return distance
	else
		return nil
	end
end

function SSToolkitEx:GetPosition(form)
 	local result = nil
 	if form ~= nil then
  		local leftAnchor, topAnchor, rightAnchor, bottomAnchor = form:GetAnchorOffsets()
  		result =
  		{
   		Bottom = bottomAnchor,
   		Left = leftAnchor,
   		Right = rightAnchor,
   		Top = topAnchor,
  		}
 	end
 	return result
end

function SSToolkitEx:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	local tSave = {}
	tSave.Enabled 				= self.opt.VPIconEnabled:IsChecked()
	tSave.Timer 				= self.opt.Timer:IsChecked()
	tSave.Size 					= self.opt.Size:GetValue()
	tSave.IgniteMark 			= self.opt.IgniteMark:IsChecked()
	tSave.IgniteTimer 			= self.opt.IgniteTimer:IsChecked()
	tSave.IgniteSize 			= self.opt.IgniteSize:GetValue()
	
	tSave.XY 					= self.opt.XY:IsChecked()
	
	tSave.IgniteFinalAlert 		= self.opt.IgniteFinalAlert:IsChecked()
	tSave.IgniteDeflectAlert 	= self.opt.IgniteDeflectAlert:IsChecked()
	tSave.IgniteSurgeAlert 		= self.opt.IgniteSurgeAlert:IsChecked()
	
	tSave.IgniteFinalSoundOpt 	= self.opt.IgniteFinalSound:IsChecked()
	tSave.IgniteDeflectSoundOpt = self.opt.IgniteDeflectSound:IsChecked()
	tSave.IgniteSurgeSoundOpt 	= self.opt.IgniteSurgeSound:IsChecked()
	
	tSave.IgniteFinalSound 		= self.opt.SoundFinal:GetText()
	tSave.IgniteDeflectSound	= self.opt.SoundDeflect:GetText()
	tSave.IgniteSurgeSound 		= self.opt.SoundSurge:GetText()
	
	tSave.IgniteList	 		= self.opt.IgniteList:IsChecked()

	tSave.PosIgniteUI			= self:GetPosition(self.IgniteUI)
	tSave.PosConfigUI			= self:GetPosition(self.wndMain)
	tSave.PosIgniteAlert		= self:GetPosition(self.opt.IgniteAlert)
	tSave.PosXY 				= self:GetPosition(self.VPUI)
	tSave.Version				= self.Version
	return tSave
end

function SSToolkitEx:OnRestore(eType, tData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	if tData.Version			~= nil then
		if tData.Version ~= self.Version and self.NeedReset then return end --Reset settings
		if tData.Version > self.Version then return end --Reset settings on downgrade
	end
	
	if tData.Enabled 				~= nil then self.OptEnabled 				= tData.Enabled 				end
	if tData.Timer 					~= nil then self.OptTimer 					= tData.Timer 					end
	if tData.Size 					~= nil then self.OptSizeVP 					= tData.Size 					end

	if tData.IgniteMark 			~= nil then self.OptIgniteMark 				= tData.IgniteMark 				end
	if tData.IgniteTimer 			~= nil then self.OptIgniteTimer 			= tData.IgniteTimer				end
	if tData.IgniteSize 			~= nil then self.OptSizeIgnite 				= tData.IgniteSize 				end
	if tData.XY 					~= nil then self.OptXY 						= tData.XY 						end
	
	if tData.IgniteFinalAlert 		~= nil then self.OptIgniteFinalAlert 		= tData.IgniteFinalAlert 		end
	if tData.IgniteDeflectAlert 	~= nil then self.OptIgniteDeflectAlert 		= tData.IgniteDeflectAlert 		end
	if tData.IgniteSurgeAlert 		~= nil then self.OptIgniteSurgeAlert		= tData.IgniteSurgeAlert 		end
	
	if tData.IgniteFinalSoundOpt 	~= nil then self.OptIgniteFinalSoundOpt 	= tData.IgniteFinalSoundOpt 	end
	if tData.IgniteDeflectSoundOpt 	~= nil then self.OptIgniteDeflectSoundOpt 	= tData.IgniteDeflectSoundOpt 	end
	if tData.IgniteSurgeSoundOpt 	~= nil then self.OptIgniteSurgeSoundOpt		= tData.IgniteSurgeSoundOpt 	end
	
	if tData.IgniteFinalSound 		~= nil then self.OptIgniteFinalSound 		= tData.IgniteFinalSound 		end
	if tData.IgniteDeflectSound 	~= nil then self.OptIgniteDeflectSound 		= tData.IgniteDeflectSound 		end
	if tData.IgniteSurgeSound 		~= nil then self.OptIgniteSurgeSound		= tData.IgniteSurgeSound 		end
	
	if tData.IgniteList 			~= nil then self.OptIgniteList				= tData.IgniteList 				end
	
	if tData.Version			~= nil then
		if tData.Version ~= self.Version and self.NeedPositionReset then return end --Reset settings
	end

	if tData.PosIgniteUI			~= nil then self.OptPosIgniteUI				= tData.PosIgniteUI 			end
	if tData.PosXY 					~= nil then self.OptPosXY 					= tData.PosXY 					end
	if tData.PosIgniteAlert			~= nil then self.OptPosIgniteAlert			= tData.PosIgniteAlert 			end

	if tData.Version			~= nil then
		if tData.Version ~= self.Version and self.NeedSettingsOnlyPositionReset then return end --Reset settings
	end
	
	if tData.PosConfigUI 			~= nil then self.OptPosConfigUI				= tData.PosConfigUI 			end
end

-----------------------------------------------------------------------------------------------
-- SSToolkitExForm Functions
-----------------------------------------------------------------------------------------------
function SSToolkitEx:PopulateSounds()
	local Button
	for key,val in pairs(Sound) do
		if val ~= nil and type(val) ~= "function" then
			Button = Apollo.LoadForm(self.xmlDoc, "SoundListItem", self.SoundList, self)
			Button:FindChild("ButtonSoundListItem"):FindChild("Text"):SetText(key)
		end
	end
	self.SoundList:ArrangeChildrenVert(0)
end

function SSToolkitEx:OnLockUnlock()
	if self.UIUnlocked == true then
		self:OnLock()
	else
		self:OnUnlock()
	end
end

function SSToolkitEx:OnVPXY()
	self.VPUI:Show(false, true)
end

function SSToolkitEx:OnUnlock()
	self.UIUnlocked = true
	self.opt.LockText:SetText("UI UNLOCKED")
	
	self:Ignite()
	self:OnIgniteBG()
	self:OnVPXY()
	
	self.VPUI:RemoveStyle("RequireMetaKeyToMove")
	self.VPUI:AddStyle("Border")
	
	self.IgniteUI:AddStyle("Sizable")
	self.IgniteUI:RemoveStyle("RequireMetaKeyToMove")
	self.IgniteUI:Show(true, true)
	
	self.opt.IgniteAlert:RemoveStyle("RequireMetaKeyToMove")
	self.opt.IgniteAlert:Show(true, true)
end

function SSToolkitEx:OnLock()
	self.UIUnlocked = false
	self.opt.LockText:SetText("UI LOCKED")
	
	self:Ignite("clear")
	self:OnIgniteBG()
	self:OnVPXY()
	
	self.VPUI:AddStyle("RequireMetaKeyToMove")
	self.VPUI:RemoveStyle("Border")
	
	self.IgniteUI:RemoveStyle("Sizable")
	self.IgniteUI:AddStyle("RequireMetaKeyToMove")
	self:OnIgnitListClick()
	
	self.opt.IgniteAlert:AddStyle("RequireMetaKeyToMove")
	self.opt.IgniteAlert:Show(false, true)
end

function SSToolkitEx:OnOK()
	self:OnSave(GameLib.CodeEnumAddonSaveLevel.Character)
	self:OnLock()
	self.OverlaySound:Close() -- hide the window
	self.wndMain:Close() -- hide the window
end

function SSToolkitEx:OnCancel()
	self:OnSave(GameLib.CodeEnumAddonSaveLevel.Character)
	self:OnLock()
	self.OverlaySound:Close() -- hide the window
	self.wndMain:Close() -- hide the window
end

function SSToolkitEx:OnButtonClose( wndHandler, wndControl, eMouseButton )
	self:OnSave(GameLib.CodeEnumAddonSaveLevel.Character)
	self:OnLock()
	self.OverlaySound:Close() -- hide the window
	self.wndMain:Close() -- hide the window
end

function SSToolkitEx:OnIgniteBG()
	if self.UIUnlocked == false then
		self.IgniteUI:RemoveStyle("Border")
	else
		self.IgniteUI:AddStyle("Border")
	end
end

function SSToolkitEx:OnSoundSelectOn()
	self.OverlaySound:Show(true, true)
	self.OverlaySound:ToFront()
end

function SSToolkitEx:OnSoundSelectOff()
	self.OverlaySound:Show(false, true)
	self.wndMain:ToFront()
end

function SSToolkitEx:OnSoundItemClick( wndHandler, wndControl, eMouseButton )
	local soundname = wndControl:FindChild("Text"):GetText()
	self.SoundCurrent:SetText(soundname)
	Sound.Play(Sound[soundname])
end

function SSToolkitEx:OnMenuButton( wndHandler, wndControl, eMouseButton )
	self:OnMenuHide()
	if wndControl:GetName() == "ButtonMenuVP" 				then self.wndMain:FindChild("GroupVP"):Show(true, true) end
	if wndControl:GetName() == "ButtonMenuIgnite" 			then self.wndMain:FindChild("GroupIgnite"):Show(true, true) end
end

function SSToolkitEx:OnMenuHide()
	self.wndMain:FindChild("GroupVP"):Show(false, true)
	self.wndMain:FindChild("GroupIgnite"):Show(false, true)
end

function SSToolkitEx:OnSoundSetFinal()
	local soundname = self.SoundCurrent:GetText()
	if Sound[soundname] == nil then return end
	self.opt.SoundFinal:SetText(soundname)
end

function SSToolkitEx:OnSoundSetDeflect()
	local soundname = self.SoundCurrent:GetText()
	if Sound[soundname] == nil then return end
	self.opt.SoundDeflect:SetText(soundname)
end

function SSToolkitEx:OnSoundSetSurge()
	local soundname = self.SoundCurrent:GetText()
	if Sound[soundname] == nil then return end
	self.opt.SoundSurge:SetText(soundname)
end

function SSToolkitEx:OnButtonTest1( wndHandler, wndControl, eMouseButton )
	--Debugging
	--local target = GameLib.GetTargetUnit()
	--Print(tostring(target:GetName()))
end

-- UI options
function SSToolkitEx:RegisterGUIElements()
	--Main
	self.opt.Version 			= 	self.wndMain:FindChild("LabelVersion")
	self.opt.LockText			=	self.wndMain:FindChild("EditLock")

	--Void Pact
	self.opt.VPIconEnabled 		= 	self.wndMain:FindChild("GroupVP"):FindChild("GroupVPtracking"):FindChild("ButtonVPIcon")
	self.opt.Size 				= 	self.wndMain:FindChild("GroupVP"):FindChild("GroupVPtracking"):FindChild("SliderBar")
	self.opt.Timer 				= 	self.wndMain:FindChild("GroupVP"):FindChild("GroupVPtracking"):FindChild("ButtonVPTimer")
	
	self.opt.XY		 			= 	self.wndMain:FindChild("GroupVP"):FindChild("GroupVPOptions"):FindChild("ButtonVPXofY")

	
	--Ignite
	self.opt.IgniteMark 		= 	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteTargetIcon"):FindChild("ButtonIgniteIcon")
	self.opt.IgniteTimer		=	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteTargetIcon"):FindChild("ButtonIgniteTimer")
	self.opt.IgniteList 		= 	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteTargetIcon"):FindChild("ButtonIgniteList")
	self.opt.IgniteSize 		= 	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteTargetIcon"):FindChild("SliderBarIgnite")
	
	self.opt.IgniteFinalAlert 	=	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteAlert"):FindChild("ButtonIgniteFinalAlert")
	self.opt.IgniteDeflectAlert =	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteAlert"):FindChild("ButtonIgniteDeflectAlert")
	self.opt.IgniteSurgeAlert 	=	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteAlert"):FindChild("ButtonIgniteSurgeAlert")
	
	self.opt.IgniteFinalSound 	=	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteSound"):FindChild("ButtonIgniteFinalSound")
	self.opt.IgniteDeflectSound =	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteSound"):FindChild("ButtonIgniteDeflectSound")
	self.opt.IgniteSurgeSound 	=	self.wndMain:FindChild("GroupIgnite"):FindChild("GroupIgniteSound"):FindChild("ButtonIgniteSurgeSound")

	self.opt.IgniteAlert		=	self.OverlayIgnite:FindChild("MainIconTextBox")
	self.opt.IgniteText 		= 	self.OverlayIgnite:FindChild("MainIconTextBox"):FindChild("Text")
	self.opt.IgniteSubText 		= 	self.OverlayIgnite:FindChild("MainIconTextBox"):FindChild("SubText")
	
	self.IgniteUI 	 			=	self.OverlayLists:FindChild("Ignite")
	self.IgniteList 			=	self.IgniteUI:FindChild("List")
	
	self.VPUI					=	self.OverlayLists:FindChild("VPcount")
	self.VPText					=	self.VPUI:FindChild("EditVP")
	
	--Sound list
	self.SoundList				=	self.OverlaySound:FindChild("ItemList")
	self.SoundCurrent			=	self.OverlaySound:FindChild("LabelCurrentSound")
	self.opt.SoundFinal			=	self.OverlaySound:FindChild("EditSoundFinal")
	self.opt.SoundDeflect		=	self.OverlaySound:FindChild("EditSoundDeflect")
	self.opt.SoundSurge			=	self.OverlaySound:FindChild("EditSoundSurge")
	
end

function SSToolkitEx:OnSliderChange( wndHandler, wndControl, fNewValue, fOldValue )
	if wndControl ~= nil then
		if wndControl:GetName() == "SliderBar" then self.opt.SizeVP = fNewValue end
		if wndControl:GetName() == "SliderBarIgnite" then self.opt.SizeIgnite = fNewValue end
	end
	
	--Adjust VP font size
	if 		self.opt.SizeIgnite <= 16 then 								self.opt.Ignitefont = "CRB_Interface7_BB"
	elseif 	self.opt.SizeIgnite <= 20 and self.opt.SizeIgnite > 16 then self.opt.Ignitefont = "CRB_Interface9_BB"
	elseif 	self.opt.SizeIgnite <= 25 and self.opt.SizeIgnite > 20 then self.opt.Ignitefont = "CRB_Interface12_BB"
	elseif 	self.opt.SizeIgnite <= 30 and self.opt.SizeIgnite > 25 then self.opt.Ignitefont = "CRB_Interface14_BB"
	else																self.opt.Ignitefont = "CRB_Interface16_BB"
	end
	
	--Adjust Ignite font size
	if 		self.opt.SizeVP <= 16 then 							self.opt.VPfont = "CRB_Interface7_BB"
	elseif 	self.opt.SizeVP <= 20 and self.opt.SizeVP > 16 then self.opt.VPfont = "CRB_Interface9_BB"
	elseif 	self.opt.SizeVP <= 25 and self.opt.SizeVP > 20 then self.opt.VPfont = "CRB_Interface12_BB"
	elseif 	self.opt.SizeVP <= 30 and self.opt.SizeVP > 25 then self.opt.VPfont = "CRB_Interface14_BB"
	else														self.opt.VPfont = "CRB_Interface16_BB"
	end
end

function SSToolkitEx:OnDirectionButton( wndHandler, wndControl, eMouseButton )
	--
end

function SSToolkitEx:OnButtonEnable( wndHandler, wndControl, eMouseButton )
	if wndControl:IsChecked() then
		self:EnableShow()
	else
		self:EnableHide()
	end
end

function SSToolkitEx:EnableShow()
	self.opt.Size:Show(true)
	self.opt.Timer:Show(true)
end

function SSToolkitEx:EnableHide()
	self.opt.Size:Show(false)
	self.opt.Timer:Show(false)
end

function SSToolkitEx:OnEnableButtonClick( wndHandler, wndControl, eMouseButton )
end


function SSToolkitEx:OnEnableButtonClick( wndHandler, wndControl, eMouseButton )
end
function SSToolkitEx:OnEnableButtonClick( wndHandler, wndControl, eMouseButton )
end

-----------------------------------------------------------------------------------------------
-- SSToolkitEx Instance
-----------------------------------------------------------------------------------------------
local SSToolkitExInst = SSToolkitEx:new()
SSToolkitExInst:Init()
